{% extends "base.html" %}

{% block title %}Scraper: Add Crawl{% endblock %}

{% block body_class %}index{% endblock %}

{% block content %}
<h1>Add Crawl</h1>

{% include 'add-or-edit-crawl.html' %}

{% endblock %}
